package com.urbandroid.common

import android.app.Notification
import android.app.PendingIntent
import android.content.Intent
import android.support.annotation.CallSuper
import android.support.annotation.DrawableRes
import android.support.v4.app.NotificationCompat
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.R
import com.urbandroid.sleep.alarmclock.GlobalInitializator


abstract class ForegroundService @JvmOverloads constructor(
        private val channelId: String,
        private val notificationId: Int,
        @DrawableRes private val smallIcon: Int = R.drawable.empty) : LoggingService()
{
    private var startForegroundCalled: Boolean = false

    @CallSuper
    override fun onCreate() {
        GlobalInitializator.initializeIfRequired(this)
    }

    @Synchronized
    protected fun startForegroundOnce(id: Int, notification: Notification) {
        if (!startForegroundCalled) {
            super.startForeground(id, notification)
            startForegroundCalled = true
            Logger.logDebug("startForegroundOnce called for "  + this.javaClass.simpleName)
        } else {
            Logger.logWarning("startForegroundOnce already called for "  + this.javaClass.simpleName)
        }
    }

    @Synchronized
    protected fun stopForegroundSelf() {
        Logger.logDebug("stopForegroundSelf called for "  + this.javaClass.simpleName + " startForegroundCalled=$startForegroundCalled")
        if (!startForegroundCalled) {
            val label = this.javaClass.simpleName
            val notificationBuilder = NotificationCompat.Builder(this, channelId).apply {
                setContentIntent(PendingIntent.getActivity(applicationContext, 0, Intent(), 0))
                color = resources.getColor(R.color.tint_dark)
                setContentTitle(label)
                setContentText("")
                setSmallIcon(smallIcon)
            }
            startForegroundOnce(notificationId, notificationBuilder.build())
        }
        stopSelf()
    }

    @Synchronized
    fun wasStartForegroundCalled() = startForegroundCalled

}