package com.urbandroid.common

import android.content.Context
import android.net.Uri
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.R
import com.urbandroid.sleep.alarmclock.DefaultRingtoneResolveService
import com.urbandroid.sleep.media.InternalRingtones
import com.urbandroid.sleep.service.Settings
import com.urbandroid.util.Md5Util
import java.text.SimpleDateFormat
import java.util.*

const val FALLBACK_RING_TONE_URI = "android.resource://com.urbandroid.sleep/raw/fallbackring"

fun String?.md5(): String? {
    if (this == null) return null
    return Md5Util.md5(this)
}

fun getActualDefaultRingtoneUri(context: Context): Uri {
    val settings = Settings(context)
    if (settings.actualDefaultRingtoneUri != null) {
        return Uri.parse(settings.actualDefaultRingtoneUri)
    }

    DefaultRingtoneResolveService.start(context)

    val list = InternalRingtones.getRingtoneList()
    // look for nature birds if not found take first and if list is empty return FALLBACK_RING_TONE_URI
    val ringTone = list.firstOrNull { it.titleRes == R.string.ringtone_nature_BIRDS }?:list.firstOrNull()
    if (ringTone == null) {
        Logger.logWarning("no internal rigntone found - selecting fallback: $FALLBACK_RING_TONE_URI")
        return Uri.parse(FALLBACK_RING_TONE_URI)
    }
    return Uri.parse(ringTone.uri)
}

@JvmOverloads
fun String.toCalendar(pattern: String = "yyyy-MM-dd HH:mm"): Calendar {
    val date = this
    return Calendar.getInstance().apply {
        time = SimpleDateFormat(pattern).parse(date)
        set(Calendar.SECOND, 0);
        set(Calendar.MILLISECOND, 0)
    }
}

@JvmOverloads
fun Calendar.format(pattern: String = "yyyy-MM-dd HH:mm"): String = SimpleDateFormat(pattern).format(time)

val Calendar.dayOfYear: Int get() = get(Calendar.DAY_OF_YEAR)