package com.urbandroid.sleep

import android.app.PendingIntent
import android.app.PendingIntent.FLAG_UPDATE_CURRENT
import android.content.Context
import android.content.Intent
import android.os.Build
import com.urbandroid.sleep.PendingIntentBuilder.Type.BROADCAST
import com.urbandroid.sleep.PendingIntentBuilder.Type.SERVICE
import com.urbandroid.util.Experiments

class PendingIntentBuilder(val context: Context, val requestCode: Int = 0, val intent: Intent, val flags: Int = FLAG_UPDATE_CURRENT) {

    enum class Type { BROADCAST, SERVICE }

    fun getBroadcast():PendingIntent? = PendingIntent.getBroadcast(context, requestCode, intent, flags)

    fun getExplicitBroadcast(): PendingIntent? {
            Experiments.initializeIfRequired(context)
            return if (Experiments.getInstance()?.isBackgroundLimitationApi26 == true && intent.hasNoPackage())
                getAlwaysExplicitBroadcast()
            else
                getBroadcast()
    }

    private fun getAlwaysExplicitBroadcast(): PendingIntent? {
        val pkgIntent = intent.copy(context.packageName)
//        if (Logger.isInitialized() && BuildConfig.BETA) Logger.logDebug("getAlwaysExplicitBroadcast(): $pkgIntent")
        return PendingIntent.getBroadcast(context, requestCode, pkgIntent, flags)
    }

    fun getService(): PendingIntent = PendingIntent.getService(context, requestCode, intent, flags)

    fun getForegroundService(): PendingIntent =
            if (Build.VERSION.SDK_INT >= 26)
                PendingIntent.getForegroundService(context, requestCode, intent, flags)
            else
                getService()

    fun cancelAlarmBroadcast() = cancelAlarm(BROADCAST)

    fun cancelAlarmService() = cancelAlarm(SERVICE)

    private fun cancelAlarm(type: Type) {
        val am = context.getAlarmManager()
//        if (Logger.isInitialized()) Logger.logDebug("Cancel Alarm $type ${intent.action}")
        when (type) {
            BROADCAST -> {
                am.cancel(getAlwaysExplicitBroadcast())
                am.cancel(getBroadcast())
            }
            SERVICE -> {
                am.cancel(getService())
                am.cancel(getForegroundService())

                if (intent.hasNoPackage()) {
                    val pkgIntent = intent.copy(context.packageName)
                    am.cancel(PendingIntent.getService(context, requestCode, pkgIntent, flags))
                    if (Build.VERSION.SDK_INT >= 26) {
                        am.cancel(PendingIntent.getForegroundService(context, requestCode, pkgIntent, flags))
                    }
                }
            }
        }
    }

    companion object {
        @JvmStatic
        @JvmOverloads
        fun get(context: Context, requestCode: Int = 0, intent: Intent, flags: Int = FLAG_UPDATE_CURRENT) = PendingIntentBuilder(context, requestCode, intent, flags)
    }
}