package com.urbandroid.sleep.activityrecognition

import android.content.Context
import android.content.Intent
import android.os.SystemClock
import android.support.v4.app.JobIntentService
import com.google.android.gms.location.ActivityTransition
import com.google.android.gms.location.ActivityTransitionEvent
import com.google.android.gms.location.ActivityTransitionResult
import com.google.android.gms.location.DetectedActivity
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.activityrecognition.ActivityIntervals.Companion.DAY_TIME_SECOND_FORMAT
import com.urbandroid.sleep.alarmclock.GlobalInitializator
import com.urbandroid.util.Experiments
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit

/**
 * IntentService for handling incoming intents that are generated as a result of requesting
 * activity transition updates
 */
class DetectedTransitionIntentService : JobIntentService() {

    override fun onHandleWork(intent: Intent) {
        GlobalInitializator.initializeIfRequired(this)
        val activities =
                when {
                    ACTION_OTHER_ACTIVITY_DETECTED == intent.action -> {
                        val timestamp = intent.getLongExtra(EXTRA_TIMESTAMP, SystemClock.elapsedRealtime() * 1_000_000)
                        listOf(
                                ActivityTransitionEvent(DetectedActivity.STILL, ActivityTransition.ACTIVITY_TRANSITION_EXIT, timestamp - TimeUnit.SECONDS.toNanos(1)),
                                ActivityTransitionEvent(DetectedActivity.STILL, ActivityTransition.ACTIVITY_TRANSITION_ENTER, timestamp)
                        )
                    }
                    ActivityTransitionResult.hasResult(intent) -> ActivityTransitionResult.extractResult(intent)?.transitionEvents?:listOf()
                    else -> listOf()
                }

        if (activities.isNotEmpty()) {
            synchronized(lock) {
                activities.forEach {
                    Logger.logDebug("activity: ${it.activity} time: ${SimpleDateFormat(DAY_TIME_SECOND_FORMAT).format(Date(it.timestamp))} transition: ${it.transition} ")

                    val file = ActivityIntervals.getStorageFile(applicationContext)
                    val intervals = ActivityIntervals
                            .from(file)
                    val size = intervals.intervals.size
                    intervals
                            .add(it)
                            .export(file)
                    if (Experiments.getInstance().isOurExperimentalPhone || size != intervals.intervals.size) {
                        Logger.logDebug("intervals ${intervals.toString(true)}")
                    }
                }
            }
        }
    }

    companion object {
        const val ACTION_OTHER_ACTIVITY_DETECTED = "com.urbandroid.sleep.activityrecognition.ACTION_OTHER_ACTIVITY_DETECTED"
        const val EXTRA_TIMESTAMP = "timestamp"
        private val lock = Object()
        private const val JOB_ID = 668
        fun start(context: Context, intent: Intent) {
            enqueueWork(context, DetectedTransitionIntentService::class.java, JOB_ID, intent)
        }
    }
}