package com.urbandroid.sleep.activityrecognition

import android.content.Context
import android.content.Intent
import android.support.v4.app.JobIntentService
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.activityrecognition.calculator.SleepTimeCalculator
import com.urbandroid.sleep.alarmclock.GlobalInitializator
import com.urbandroid.sleep.service.SharedApplicationContext
import com.urbandroid.sleep.settings
import com.urbandroid.util.Experiments

class SleepTimeSuggestionNotificationService : JobIntentService() {

    override fun onHandleWork(intent: Intent) {
        GlobalInitializator.initializeIfRequired(this)

        if (applicationContext.settings.sleepTimeSuggestionMode == SleepTimeSuggestionManager.Mode.DISABLED) return

        val from = SleepTimeCalculator.from().timeInMillis
        val to = SleepTimeCalculator.to().timeInMillis

        val hasRecord = SharedApplicationContext.getInstance().sleepRecordRepository
                .getSleepRecords(0, 4, false)
                .any { (it.from != null && it.from.time in from..to) || (it.to != null && it.to.time in from..to) }

        Logger.logDebug("SleepTimeSuggestionNotificationService: has record in ${ActivityIntervals.formatWithDay(from)} - ${ActivityIntervals.formatWithDay(to)}: $hasRecord")

        if (!hasRecord || Experiments.getInstance().isOurExperimentalPhone) {
            SleepTimeSuggestionManager(applicationContext).suggest(settings.sleepTimeSuggestionMode)
            Logger.logDebug("estimate notification called")
        }

    }

    companion object {
        private const val JOB_ID = 31415
        fun start(context: Context, intent: Intent) {
            enqueueWork(context, SleepTimeSuggestionNotificationService::class.java, JOB_ID, intent)
        }
    }
}