package com.urbandroid.sleep.activityrecognition.calculator

import com.urbandroid.sleep.activityrecognition.ActivityIntervals
import com.urbandroid.sleep.activityrecognition.ActivityIntervals.Companion.format
import java.util.*
import java.util.concurrent.TimeUnit
import java.util.concurrent.TimeUnit.*

interface SleepTimeCalculator {

    fun estimate(estimate: Estimate): Estimate

    data class Estimate @JvmOverloads constructor(val type: Type, val from: Calendar, val to: Calendar, val intervals: List<ActivityIntervals.Interval> = listOf()) {
        enum class Type {
            BASIC, DB, ACTIVITY
        }
        fun expand(value: Int, unit: TimeUnit = HOURS): Estimate {
            when (unit) {
                HOURS       -> { from.add(Calendar.HOUR_OF_DAY, -value); to.add(Calendar.HOUR_OF_DAY, value) }
                MINUTES     -> { from.add(Calendar.MINUTE, -value); to.add(Calendar.MINUTE, value) }
                DAYS        -> { from.add(Calendar.DAY_OF_YEAR, -value); to.add(Calendar.DAY_OF_YEAR, value) }
                NANOSECONDS -> { from.add(Calendar.MILLISECOND, -value * 1000000); to.add(Calendar.MILLISECOND, value * 1000000) }
                SECONDS     -> { from.add(Calendar.SECOND, -value); to.add(Calendar.SECOND, value) }
                MICROSECONDS -> { from.add(Calendar.MILLISECOND, -value * 1000); to.add(Calendar.MILLISECOND, value * 1000) }
                MILLISECONDS -> { from.add(Calendar.MILLISECOND, -value); to.add(Calendar.MILLISECOND, value) }
            }
            return this
        }

        override fun toString(): String =
                        "Estimate $type [${format(from.timeInMillis)}, ${format(to.timeInMillis)}]" +
                        if (intervals.isEmpty()) "" else " intervals: ${(ActivityIntervals.from(intervals).toString(true))}"
    }

    companion object {
        const val DAY: Long = 86400000

        fun from(): Calendar = Calendar.getInstance().apply {
            add(Calendar.DAY_OF_YEAR, -1)
            set(Calendar.HOUR_OF_DAY, 20)
            set(Calendar.MINUTE, 0)
            set(Calendar.SECOND, 0)
            set(Calendar.MILLISECOND, 0)
        }

        fun to(): Calendar = Calendar.getInstance().apply {
            set(Calendar.HOUR_OF_DAY, 7)
            set(Calendar.MINUTE, 0)
            set(Calendar.SECOND, 0)
            set(Calendar.MILLISECOND, 0)
        }
    }

}