package com.urbandroid.sleep.alarmclock

import android.content.Context
import android.content.Intent
import android.media.RingtoneManager
import android.support.v4.app.JobIntentService
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.service.Settings
import kotlin.system.measureTimeMillis

class DefaultRingtoneResolveService : JobIntentService() {

    override fun onHandleWork(intent: Intent) {
        GlobalInitializator.initializeIfRequired(applicationContext)
        val settings = Settings(applicationContext)
        val t = measureTimeMillis {
            if (settings.actualDefaultRingtoneUri == null) {
                try {
                    val uri = RingtoneManager.getActualDefaultRingtoneUri(applicationContext, RingtoneManager.TYPE_ALARM)?.toString()
                    settings.actualDefaultRingtoneUri = uri
                } catch (ex: Exception) {
                    Logger.logWarning("getActualDefaultRingtoneUri", ex)
                }
            }
        }
        Logger.logDebug("uri resolved ${settings.actualDefaultRingtoneUri} in $t ms")
    }

    companion object {
        private const val JOB_ID = 668
        private const val ACTION_DEFAULT_RINGTONE_RESOLVE = "com.urbandroid.sleep.alarmclock.ACTION_DEFAULT_RINGTONE_RESOLVE"

        @JvmStatic
        fun start(context: Context) {
            enqueueWork(context, DefaultRingtoneResolveService::class.java, JOB_ID, Intent(ACTION_DEFAULT_RINGTONE_RESOLVE))
        }
    }
}

