package com.urbandroid.sleep.alarmclock

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import com.urbandroid.sleep.SleepService
import com.urbandroid.sleep.media.lullaby.LullabyService

/**
 * This is helper class for re-wrapping explicit intent actions to implicit and broadcasts them to 3rd party applications
 */
private class ThirdPartyImplicitIntentRewrapper : BroadcastReceiver() {

    override fun onReceive(context: Context?, intent: Intent?) {
        if (context == null || intent == null || intent.action == null) {
            return
        }
        context.sendBroadcast(Intent(intent.action + POSTFIX))
    }

    companion object {
        private const val POSTFIX = "-IMPLICIT"

        fun register(context: Context) {
            context.registerReceiver(ThirdPartyImplicitIntentRewrapper(),
                    IntentFilter().apply {
                        addAction(SleepService.ACTION_SLEEP_TRACKING_STARTED)
                        addAction(SleepService.ACTION_SLEEP_TRACKING_STOPPED)
                        addAction(Alarms.ALARM_SNOOZE_CLICKED_ACTION)
                        addAction(Alarms.TIME_TO_BED_ALERT_ACTION)
                        addAction(Alarms.ALARM_ALERT_START)
                        addAction(Alarms.ALARM_ALERT_DISMISS)
                        addAction(LullabyService.ACTION_LULLABY_START_PLAYBACK)
                        addAction(LullabyService.ACTION_LULLABY_STOPPED_PLAYBACK)
                    }
            )
        }
    }

}