/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. THIS FILE HAS BEEN MODIFIED
 */

package com.urbandroid.sleep.alarmclock.settings

import android.app.Activity
import android.content.Intent
import android.media.AudioManager
import android.os.Build
import android.preference.ListPreference
import android.preference.Preference
import android.preference.PreferenceGroup
import com.urbandroid.sleep.R
import com.urbandroid.sleep.gui.PreferenceActivity
import com.urbandroid.sleep.media.MediaListActivity
import com.urbandroid.sleep.media.MediaListActivity.*
import com.urbandroid.sleep.notification.NOTIFICATION_CHANNEL_ID_BEDTIME
import com.urbandroid.sleep.notification.showSettings
import com.urbandroid.sleep.service.SettingKeys
import com.urbandroid.sleep.service.SharedApplicationContext

class BedtimeSettingsActivity : SimpleSettingsActivity() {
    override fun onResume() {
        super.onResume()
        if (supportActionBar != null) {
            supportActionBar?.setTitle(R.string.target_sleep_time_notify_title)
        }
    }

    override fun getSettings(): Int {
        return R.xml.settings_bedtime
    }

    override fun refresh(preferenceActivity: PreferenceActivity, searchMode: Boolean) {
        val preferenceScreen = preferenceActivity.preferenceScreen

        val pref = preferenceScreen.findPreference(SettingKeys.TARGET_SLEEP_TIME_NOTIFY_BEFORE) as ListPreference
        val channelPref = preferenceScreen.findPreference(SettingKeys.KEY_TIME_TO_BED_CHANNEL)
        val ringTonePref = preferenceScreen.findPreference(SettingKeys.KEY_TIME_TO_BED_RINGTONE)
        val vibratePref = preferenceScreen.findPreference(SettingKeys.KEY_VIBRATING_TIME_TO_BED)
        val ledPref = preferenceScreen.findPreference(SettingKeys.KEY_TIME_TO_BED_LED)

        val notificationCategory = preferenceScreen.findPreference(SettingKeys.KEY_CATEGORY_TIME_TO_BED_NOTIFICATION) as PreferenceGroup

        if (pref != null) {
            pref.summary = pref.entry
            pref.onPreferenceChangeListener = this
        }

        if (ringTonePref != null) {
            ringTonePref.onPreferenceClickListener = Preference.OnPreferenceClickListener {
                val intent = Intent(preferenceActivity, MediaListActivity::class.java).apply {
                    putExtra(EXTRA_MULTIPLE, false)
                    putExtra(EXTRA_STREAM, AudioManager.STREAM_NOTIFICATION)
                    putExtra(EXTRA_IS_PER_ALARM, true)
                    putExtra(EXTRA_RADIO, false)
                    putExtra(EXTRA_URI, SharedApplicationContext.getSettings().timeToBedRingtone)
                }
                preferenceActivity.startActivityForResult(intent, RINGTONE_RESULT)
                true
            }
        }

        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O) {
            if (channelPref != null) {
                notificationCategory.removePreference(channelPref)
            }
        } else {
            if (vibratePref != null) {
                notificationCategory.removePreference(vibratePref)
            }
            if (ledPref != null) {
                notificationCategory.removePreference(ledPref)
            }
            if (ringTonePref != null) {
                notificationCategory.removePreference(ringTonePref)
            }
            channelPref.onPreferenceClickListener = Preference.OnPreferenceClickListener {
                showSettings(applicationContext, NOTIFICATION_CHANNEL_ID_BEDTIME)
                true
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (data != null && requestCode == RINGTONE_RESULT && resultCode == Activity.RESULT_OK) {
            findPreference<Preference>(SimpleSettingsActivity.KEY_TIME_TO_BED_RINGTONE)?.summary = data.getStringExtra(EXTRA_TITLE)
            SharedApplicationContext.getSettings().timeToBedRingtone = data.getStringExtra(EXTRA_URI)
        }

        super.onActivityResult(requestCode, resultCode, data)
    }

    override fun getDocumentationUrl(): String {
        return super.getBaseDocumentationUrl() + "core/alarms/#bedtime"
    }

    companion object {
        private val RINGTONE_RESULT = 6543
    }

}
