package com.urbandroid.sleep.domain.addon

import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Resources
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.R
import com.urbandroid.sleep.TrialFilter
import com.urbandroid.sleep.addon.stats.util.ResourceUtil
import com.urbandroid.sleep.service.SharedApplicationContext

class AddonApp(val icon: Int, private val title: String?, val desc: String?, val link: String?) : AddonItem {

    var isInstalled = false
    var isResourceIcon = false

    override fun getTitle(): String? {
        return title
    }

    override fun toString(): String {
        return title + " " + link
    }

    companion object {

        fun random(context: Context) = from(context, context.resources.getStringArray(R.array.addons_entries)).shuffled().firstOrNull() as AddonApp?

        @JvmOverloads
        fun from(context: Context, keys: Array<String>, filterInstalled: Boolean = true): List<AddonItem> {
            val items = mutableListOf<AddonApp>()
            for (key in keys) {
                val res = context.resources
                val title = getString(context, res, key, "title")
                var desc = getString(context, res, key, "desc")
                val pack = getString(context, res, key, "package")
                var link = getString(context, res, key, "link")
                val resDrawable = getString(context, res, key, "res")

                if ("addons_hue" == key) {
                    if (SharedApplicationContext.getSettings().isPhilipsOffer) {
                        desc =  if (TrialFilter.getInstance().isTrial)
                                    context.resources.getString(R.string.purchase_hue_advocate) + ".\n\n" + context.resources.getString(R.string.philips_hue_offer) + "."
                                else
                                    context.resources.getString(R.string.purchase_hue_advocate) + ".\n\n" + context.resources.getString(R.string.philips_hue_offer_unlocked) + "."
                        link = if (TrialFilter.getInstance().isTrial)
                                    context.resources.getString(R.string.philips_hue_offer_link)
                               else
                                    context.resources.getString(R.string.philips_hue_offer_link_unlocked)
                    } else {
                        continue
                    }
                }

                val signaturesCheckResult = context.packageManager.checkSignatures(context.packageName, pack)
                val installed = signaturesCheckResult == PackageManager.SIGNATURE_MATCH

                if (link == null) {
                    if (TrialFilter.getInstance().version == TrialFilter.Version.TRIAL) {
                        link = "market://details?id=" + pack!!
                    } else if (TrialFilter.getInstance().version == TrialFilter.Version.AMAZON && key.startsWith("addons")) {
                        link = "amzn://apps/android?p=" + pack!!
                    }
                }

                var isResourceIcon = false
                var icon: Int
                if (resDrawable != null) {
                    icon = ResourceUtil.getResourceByName(R.drawable::class.java, resDrawable)
                    isResourceIcon = true
                } else {
                    icon = ResourceUtil.getResourceByName(R.drawable::class.java, key + "_icon")
                }

                val app = AddonApp(icon, title, desc, link)
                app.isResourceIcon = isResourceIcon
                app.isInstalled = installed

                if (!installed || !filterInstalled) {
                    items.add(app)
                }
                Logger.logInfo("App " + app)
            }

            return items.toList()
        }

        private fun getString(context: Context, res: Resources, addonKey: String, suffix: String): String? {
            val resId = res.getIdentifier(addonKey + "_" + suffix, "string", context.packageName)
            return if (resId > 0) res.getString(resId) else null
        }
    }
}


