package com.urbandroid.sleep.fragment.addon

import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.urbandroid.common.logging.Logger
import com.urbandroid.common.util.intent.ViewIntent
import com.urbandroid.sleep.R
import com.urbandroid.sleep.TrialFilter
import com.urbandroid.sleep.domain.addon.AddonApp
import com.urbandroid.sleep.service.SharedApplicationContext

class AddOnBinder {

    fun bind(context: Context, app: AddonApp, image: ImageView, title: TextView, desc: TextView, button: Button, iconSize: Int, padding: Int) {

        desc.text = app.desc
        with (image) {
            setImageResource(app.icon)
            layoutParams.height = iconSize
            layoutParams.width = iconSize
            if (app.isResourceIcon) {
                setPadding(padding, padding, padding, padding)
            } else {
                setPadding(0, 0, 0, 0)
            }
        }

        if (app.link == null) {
            button.visibility = View.GONE
        } else {
            button.visibility = View.VISIBLE
            if (!app.link.startsWith("http")) {
                button.setText(if (app.isInstalled) R.string.addons_run else R.string.install_button)
            } else {
                button.setText(R.string.addons_install)
            }
        }

        button.setOnClickListener { view ->
            val url = app.link
            if (url != null && (url.startsWith(ViewIntent.PLAY_STORE_PREFIX) || url.startsWith(ViewIntent.AMAZON_STORE_PREFIX))) {
                val packageName = ViewIntent.packageFromMarketUrl(url)
                Logger.logInfo("Add-on clicked: $packageName url $url")
                val packageManager = context.packageManager
                val signaturesCheckResult = context.packageManager.checkSignatures(context.packageName, packageName)
                val installed = signaturesCheckResult == PackageManager.SIGNATURE_MATCH
                if (!installed) {
                    SharedApplicationContext.getInstance().firebaseAnalyticsManager.setEventAddonClicked(packageName)
                }

                try {
                    val intent = packageManager.getLaunchIntentForPackage(packageName)
                    if (intent == null) {
                        view.context.startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
                    } else {
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        context.startActivity(intent)
                    }
                } catch (e: Exception) {
                    try {
                        view.context.startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
                    } catch (e1: Exception) {
                        Toast.makeText(view.context, "Cannot open " + url, Toast.LENGTH_SHORT).show()
                    }

                }

            } else if (url != null && url.startsWith("pebble")) {
                Logger.logInfo("Add-on clicked: " + " url " + url + " gear " + TrialFilter.getInstance().isGear)
                when {
                    TrialFilter.getInstance().isPebble -> ViewIntent.url(context, "pebble://appstore/52b77e12dfa4228e8200000a")
                    TrialFilter.getInstance().isGear -> ViewIntent.market(context, "com.urbandroid.sleep.addon.generic.samsung")
                    else -> ViewIntent.url(context, "https://sleep.urbandroid.org/documentation/integration/wearable/")
                }
            } else {
                ViewIntent.url(context, url)
            }
        }

        title.text = app.title
    }

}