package com.urbandroid.sleep.fragment.dashboard.card

import android.app.Activity
import android.os.Build
import android.support.v7.widget.CardView
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.urbandroid.sleep.R
import com.urbandroid.sleep.addon.stats.goal.GoalDetailActivity
import com.urbandroid.sleep.domain.goal.Goal
import com.urbandroid.sleep.service.Settings

class GoalCard(activity: Activity) : DashboardCard<GoalCard.GoalViewHolder>(activity, DashboardCard.Type.GOAL, R.layout.card_goal) {

    override fun createViewHolder(v: View): GoalViewHolder {
        return GoalViewHolder(v)
    }

    inner class GoalViewHolder(view: View) : LayeredViewHolder(view) {
        val goalView = view.findViewById<ViewGroup>(R.id.goal)
        val title = view.findViewById<TextView>(R.id.title)
        val text = view.findViewById<TextView>(R.id.text)
        val card = view.findViewById<CardView>(R.id.foreground)
    }

    override fun onCardClicked(view : View) {
        GoalDetailActivity.start(context)
    }

    override fun getNameResource(): Int {
        return R.string.goal
    }

    override fun bindView(viewHolder: GoalViewHolder) {

        val settings = Settings(context)
        val goal = settings.currentGoal

        goal?.let {
            if (goal.updateStatus()) {
                settings.currentGoal = goal
            }
        }

        refresh(viewHolder, goal)
    }

    private fun refresh(viewHolder: GoalViewHolder, goal: Goal?) {
        if (goal != null) {
            setCardVisibility(context, viewHolder.itemView as ViewGroup, true)
            GoalBinder().bind(context, goal, viewHolder.title, viewHolder.text, viewHolder.goalView)

            if (Build.VERSION.SDK_INT >= 21) {
                viewHolder.goalView.elevation = 0f;
            }
            viewHolder.goalView.setBackgroundResource(R.drawable.circle_badge_white);
//            viewHolder.goalView.background = null;
        } else {
            setCardVisibility(context, viewHolder.itemView as ViewGroup, false)
        }

    }

    override fun isThemeCard(): Boolean {
        return false
    }

}

