package com.urbandroid.sleep.fragment.dashboard.card

import android.app.Activity
import android.content.Intent
import android.support.v4.app.ActivityCompat
import android.support.v4.app.ActivityOptionsCompat
import android.support.v4.util.Pair
import android.view.View
import com.urbandroid.sleep.R
import com.urbandroid.sleep.domain.Noise
import com.urbandroid.sleep.domain.tag.Tag
import com.urbandroid.sleep.media.PlayAudioActivity
import com.urbandroid.util.ColorUtil

class InterestingNoiseCard(activity : Activity, private val noises : List<Noise>) : DashboardCard<LastSleepRecordCard.SleepRecordViewHolder>(activity, DashboardCard.Type.SLEEP_TALK, R.layout.card_interesting_noise) {

    override fun onCardClicked(view: View) {
        if (noises.isNotEmpty()) run {
            val idArray = LongArray(noises.size)
            for (j in idArray.indices) {
                idArray[j] = noises.get(j).id!!
            }
            val intent = Intent(context, PlayAudioActivity::class.java)
            intent.putExtra(PlayAudioActivity.NOISE_IDS, idArray)
            val p1 = Pair.create(view.findViewById<View>(R.id.foreground), "box")
            val p2 = Pair.create(view.findViewById<View>(R.id.play_button), "fab")
            val option = ActivityOptionsCompat.makeSceneTransitionAnimation(context, p1, p2)
            ActivityCompat.startActivity(context, intent, option.toBundle())

        }
    }

    override fun getNameResource(): Int {
        return R.string.interesting_noise_recordings
    }


    override fun bindView(viewHolder: LastSleepRecordCard.SleepRecordViewHolder?) {
        viewHolder?.title?.setText(R.string.interesting_noise_recordings)

        val snoreCount = noises.filter {
            Tag.hasTag(it.comment, Tag.SNORE)
        }.count()

        val talkCount = noises.filter {
            Tag.hasTag(it.comment, Tag.TALK)
        }.count()

        noises.sortedWith(compareBy({Tag.hasTag(it.comment, Tag.TALK)}, {it.fromTime}))

        val snoreString = context.resources.getString(R.string.stats_caption_snore)
        val talkString = context.resources.getString(R.string.sleep_talk)

        viewHolder?.subtitle?.text = "${if (talkCount > 0) {"${context.resources.getString(R.string.last_night)}\n${talkCount}x $talkString "} else ""}${if (snoreCount > 0) {"${snoreCount}x $snoreString"} else ""}"

        if (noises.isNotEmpty()) {
            viewHolder?.graph?.let {
                it.setRotateYAxisLabels(false)
                it.setDoGradient(false)
                it.setDrawYAxis(false)
                it.setDrawAverage(false)
                it.setDoTrimming(false)
                it.setDoDrawGraphLine(false)
                it.setDrawXAxisBars(false)
                it.setForceXAxisOffset(false)
                it.setDoMirrorGraph(true)
                it.setEquidistantValues(noises[0].data, true)
                it.setGradientFullColor(ColorUtil.i(context, R.color.transparent))
                it.setGradientTransColor(ColorUtil.i(context, R.color.transparent))
                it.setSleepGraphColor(ColorUtil.i(context, R.color.white_tertiary))
            }
        }

    }

    override fun createViewHolder(v: View): LastSleepRecordCard.SleepRecordViewHolder {
        return LastSleepRecordCard.SleepRecordViewHolder(v)
    }

    override fun isThemeCard(): Boolean {
        return false
    }
}
