package com.urbandroid.sleep.fragment.dashboard.card

import android.app.Activity
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.RatingBar
import android.widget.TextView
import com.google.android.gms.ads.AdListener
import com.google.android.gms.ads.AdLoader
import com.google.android.gms.ads.VideoController
import com.google.android.gms.ads.formats.*
import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.R
import com.urbandroid.sleep.ads.SleepAdRequestBuilder


class NativeAdCard(activity: Activity) : PageAwareDashboardCard<NativeAdCard.NativeAdViewHolder>(activity, DashboardCard.Type.NATIVE_AD, R.layout.card_native_ad) {

    override fun createViewHolder(v: View): NativeAdViewHolder {
        return NativeAdViewHolder(v)
    }

    inner class NativeAdViewHolder(view: View) : LayeredViewHolder(view)

    override fun onCardClicked(view: View) {
    }

    override fun getNameResource(): Int {
        return R.string.no_ads;
    }

    override fun bindView(viewHolder: NativeAdViewHolder) {

        setCardVisibility(context, viewHolder.itemView as ViewGroup, false)

        val parent = viewHolder.itemView.findViewById(R.id.foreground) as ViewGroup

        val adLoader = AdLoader.Builder(context, "ca-app-pub-1970766161278534/6966027598")
                .forAppInstallAd { ad : NativeAppInstallAd ->
                    Logger.logInfo("AD: app install loaded ")
                    val inflater = parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                    val adView = (inflater.inflate(R.layout.card_native_ad_install, parent, false)).findViewById<NativeAppInstallAdView>(R.id.native_ad)

                    val headlineView = adView.findViewById<TextView>(R.id.nativead_headline)
                    headlineView.text = ad.headline
                    adView.headlineView = headlineView

                    val bodyView = adView.findViewById<TextView>(R.id.nativead_body)
                    bodyView.text = ad.body
                    adView.bodyView = bodyView

                    val callToActionView = adView.findViewById<TextView>(R.id.nativead_call_to_action)
                    callToActionView.text = ad.callToAction
                    adView.callToActionView = callToActionView

                    val appIconView = adView.findViewById<ImageView>(R.id.nativead_app_icon)
                    if (ad.icon != null) {
                        appIconView.setImageDrawable(ad.icon.drawable)
                        adView.iconView = appIconView
                        appIconView.visibility = View.VISIBLE
                    } else {
                       appIconView.visibility = View.GONE
                    }

                    val storeView = adView.findViewById<TextView>(R.id.nativead_store)

                    if (ad.store != null) {
                        storeView.text = ad.store
                        adView.storeView = storeView
                        storeView.visibility = View.VISIBLE
                    } else {
                        storeView.visibility = View.GONE
                    }

                    val priceView = adView.findViewById<TextView>(R.id.nativead_price)
                    if (ad.price != null && ad.price.trim().isNotEmpty()) {
                        priceView.text = ad.price
                        adView.priceView = priceView
                        priceView.visibility = View.VISIBLE
                    } else {
                        priceView.visibility = View.GONE
                    }

                    val starsView = adView.findViewById<RatingBar>(R.id.nativead_stars)

                    if (ad.starRating != null) {
                        starsView.rating = ad.starRating.toFloat()
                        adView.starRatingView = starsView
                        starsView.visibility = View.VISIBLE
                    } else {
                        starsView.visibility = View.GONE
                    }

                    val mediaView = adView.findViewById<MediaView>(R.id.nativead_media)
                    val mainImageView = adView.findViewById<ImageView>(R.id.nativead_image)

                    val vc = ad.videoController

                    if (vc.hasVideoContent()) {
                        adView.mediaView = mediaView
                        mainImageView.setVisibility(View.GONE)
                        mediaView.setVisibility(View.VISIBLE)
                        vc.videoLifecycleCallbacks = object : VideoController.VideoLifecycleCallbacks() {
                            override fun onVideoEnd() {
                                super.onVideoEnd()
                                mediaView.invalidate()
                            }
                        }
                    } else {
                        adView.imageView = mainImageView
                        mediaView.setVisibility(View.GONE)

                        // At least one image is guaranteed.
                        val images = ad.getImages()

                        if (images.size > 0) {
                            mainImageView.setImageDrawable(images.get(0).getDrawable())
                            mainImageView.visibility = View.VISIBLE
                        }
                    }
                    
                    adView.setNativeAd(ad)
                    parent.removeAllViews();
                    parent.addView(adView)
                    setCardVisibility(context, viewHolder.itemView, true)
                }
                .forContentAd { ad : NativeContentAd ->
                    Logger.logInfo("AD: app content loaded ")

                    val inflater = parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                    val adView = (inflater.inflate(R.layout.card_native_ad_content, parent, false)).findViewById<NativeContentAdView>(R.id.native_ad)

                    val headlineView = adView.findViewById<TextView>(R.id.nativead_headline)
                    headlineView.text = ad.headline
                    adView.headlineView = headlineView

                    val bodyView = adView.findViewById<TextView>(R.id.nativead_body)
                    bodyView.text = ad.body
                    adView.bodyView = bodyView

                    val callToActionView = adView.findViewById<TextView>(R.id.nativead_call_to_action)
                    callToActionView.text = ad.callToAction
                    adView.callToActionView = callToActionView

                    val advertizerView = adView.findViewById<TextView>(R.id.nativead_advertiser)
                    if (ad.advertiser != null) {
                        advertizerView.text = ad.advertiser
                        adView.advertiserView = advertizerView
                        advertizerView.visibility = View.VISIBLE
                    } else {
                        advertizerView.visibility = View.GONE
                    }

                    val mediaView = adView.findViewById<MediaView>(R.id.nativead_media)
                    val mainImageView = adView.findViewById<ImageView>(R.id.nativead_image)

                    val vc = ad.videoController

                    if (vc.hasVideoContent()) {
                        adView.mediaView = mediaView
                        mainImageView.setVisibility(View.GONE)
                        mediaView.setVisibility(View.VISIBLE)
                    } else {
                        adView.imageView = mainImageView
                        mediaView.setVisibility(View.GONE)

                        // At least one image is guaranteed.
                        val images = ad.getImages()

                        if (images.size > 0) {
                            mainImageView.setImageDrawable(images.get(0).getDrawable())
                            mainImageView.visibility = View.VISIBLE
                        }
                    }

                    var logoView = adView.findViewById<ImageView>(R.id.nativead_logo);

                    if (ad.logo != null) {
                        adView.logoView = logoView;
                        logoView.setImageDrawable(ad.logo.drawable);
                        logoView.visibility = View.VISIBLE
                    } else {
                        logoView.visibility = View.GONE
                    }

                    adView.setNativeAd(ad)
                    parent.removeAllViews();
                    parent.addView(adView)
                    setCardVisibility(context, viewHolder.itemView, true)
                }
                .withAdListener(object : AdListener() {
                    override fun onAdFailedToLoad(errorCode: Int) {
                        Logger.logInfo("AD: failed " + Int)
                    }
                })
                .withNativeAdOptions(NativeAdOptions.Builder().build())
                .build()

        adLoader.loadAd(SleepAdRequestBuilder.build(context));
    }

    override fun setRemoved(removed: Boolean) {
        // do nothing
    }

    override fun isRemovable(): Boolean {
        return false
    }

    override fun isPinnable(): Boolean {
        return super.isPinnable()
    }

    override fun next() {
        refresh();
    }

    override fun prev() {
        next();
    }
}
