package com.urbandroid.sleep.fragment.dashboard.card

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.support.v7.widget.CardView
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.RelativeLayout
import android.widget.TextView
import com.urbandroid.common.util.ActivityUtils
import com.urbandroid.common.util.DateUtil
import com.urbandroid.sleep.NewAddRecordActivity
import com.urbandroid.sleep.R
import com.urbandroid.sleep.alarmclock.AlarmClock
import com.urbandroid.sleep.alarmclock.Alarms
import com.urbandroid.sleep.alarmclock.SleepStarter
import com.urbandroid.sleep.sendExplicitBroadcast
import com.urbandroid.sleep.service.SharedApplicationContext

class ShortcutCard(activity : Activity) : DashboardCard<ShortcutCard.ShortcutViewHolder>(activity, DashboardCard.Type.SHORTCUT, R.layout.card_shortcut) {

    var expanded : Boolean = false;

    override fun onCardClicked(view: View) {
        val params = view.findViewById<ViewGroup>(R.id.flow)?.layoutParams as RelativeLayout.LayoutParams

        if (params.height == ActivityUtils.getDip(context, 44)) {
            params.height = ViewGroup.LayoutParams.WRAP_CONTENT
            expanded = true
        } else {
            params.height = ActivityUtils.getDip(context, 44)
            expanded = false
        }

        view.findViewById<ViewGroup>(R.id.flow)?.layoutParams = params
    }

    override fun getNameResource(): Int {
        return R.string.shortcut
    }

    private fun buildButton(context : Context) : Button {
        return Button(context, null, R.style.Buttonsleep)
    }

    override fun bindView(viewHolder: ShortcutViewHolder?) {
        viewHolder?.title?.setText(R.string.shortcut)

        var button = viewHolder?.itemView?.findViewById<Button>(R.id.shortcut_1)
        button?.text = context.resources.getString(R.string.sleep) + " " + DateUtil.formatMinutesInHumanLanguage(context, SharedApplicationContext.getSettings().idealSleepMinutes)
        button?.setOnClickListener(View.OnClickListener {
            context.sendExplicitBroadcast(Intent(Alarms.START_SLEEP_TRACK_WITH_IDEAL_ALARM_ACTION))
        })

        button = viewHolder?.itemView?.findViewById<Button>(R.id.shortcut_2)
        button?.text = context.resources.getString(R.string.add_record_title)
        button?.setOnClickListener(View.OnClickListener {
            NewAddRecordActivity.start(context)
        })
        
        val lastNapMinutes = SharedApplicationContext.getSettings().lastNapMinutes

        var count = 0

        val shortcutArray = intArrayOf(R.id.shortcut_3, R.id.shortcut_4)

        for (min in lastNapMinutes) {
            button = viewHolder?.itemView?.findViewById<Button>(shortcutArray[count])
            button?.text = AlarmClock.getLastNapText(context, min)
            button?.setOnClickListener(View.OnClickListener {
                Alarms.addQuickAlarm(context, min, context.getResources().getString(R.string.nap_alarm_label) + " " + DateUtil.formatMinutes(min) + "", true)
                SleepStarter().startSleep(context)
            })
            button?.visibility = View.VISIBLE
            count++
            if (count >= shortcutArray.size) break
        }

    }

    override fun createViewHolder(v: View): ShortcutCard.ShortcutViewHolder {
        return ShortcutCard.ShortcutViewHolder(v)
    }

    class ShortcutViewHolder(view: View) : LayeredViewHolder(view) {

        var flow: ViewGroup
        var title: TextView
        var card: CardView? = null

        init {
            card = view.findViewById(R.id.foreground)
            title = view.findViewById(R.id.title)
            flow = view.findViewById(R.id.flow)
        }

    }


    override fun isThemeCard(): Boolean {
        return true
    }
}
