package com.urbandroid.sleep.nearby.core

import android.content.Context
import com.urbandroid.common.logging.Logger
import com.urbandroid.util.StringBufferPersister
import org.json.JSONObject
import java.util.*
import kotlin.concurrent.thread

//Just a demo code to test one-to-one communication via nearby API. Not used in the real app logic.
class NearbyDemo(context: Context) {

    @Volatile
    private var stop = false

    private val fileLog = StringBufferPersister("nearby", 100, true)

    init {
        thread(start = true) {

            val logger = MyLogger { level, message, error ->
                if (level > Logger.DEBUG_LEVEL) {
                    val thread = Thread.currentThread().name
                    val errorMessage = if (error == null) "" else "${error.javaClass.simpleName} ${error.message}"
                    fileLog.update("${Date()} $thread $message $errorMessage\n")
                }
            }

            val messageConsumer: (Message) -> Unit = {
                fileLog.update("message," +
                        "${it.serialNo}," +
                        "${it.payload.getLong("time")}," +
                        "${it.payload.getDouble("activity")}\n")
            }

            ReliableMessageQueue(getServiceName("qwerty"), context, logger, messageConsumer).use {
                while(!stop) {
                    val json = JSONObject()
                    json.put("time", System.currentTimeMillis())
                    json.put("activity", Math.random())
                    it.sendReliable(json)
                    Thread.sleep(9000)
                }
            }
        }
    }

    fun close() {
        stop = true
        fileLog.flush()
    }
}
