package com.urbandroid.sleep.nearby.core

import android.content.Context
import java.util.*

private val serviceNamePrefix = "com.urbandroid.sleep.nearby"

private val rendezvousServiceName = "$serviceNamePrefix.rendezvous"

fun getServiceName(sharedSecret: String): String {
    return "$serviceNamePrefix.$sharedSecret"
}

/**
 * Connect a counterparty on the known rendezvous address and agree on a shared secret token
 * that will be used as a suffix of the private address between the two points.
 */
fun rendezvous(context: Context,
               logger: MyLogger,
               timeoutMillis: Long)
        : String {

    val start = System.currentTimeMillis()

    val myToken = UUID.randomUUID().toString()
    var myTokenConfirmed = false
    var remoteToken = ""

    var endpoint: Endpoint? = null;

    val payloadConsumer: (ByteArray) -> Unit = {
        val receivedToken = String(it)
        if (receivedToken == myToken) {
            myTokenConfirmed = true
        } else {
            remoteToken = receivedToken
            endpoint!!.send(receivedToken.toByteArray())
        }
    }

    endpoint = Endpoint(rendezvousServiceName, context, logger, payloadConsumer)

    endpoint.use {
        while(  System.currentTimeMillis() - start < timeoutMillis &&
                (remoteToken.isEmpty() || !myTokenConfirmed))
        {
            Thread.sleep(500)
            if (it.isConnected()) {
                it.send(myToken.toByteArray())
            }
        }
    }

    if (remoteToken.isEmpty() || !myTokenConfirmed) {
        return ""
    }

    val sharedToken = if (myToken < remoteToken) myToken else remoteToken

    return sharedToken
}
