package com.urbandroid.sleep.nearby.pairtracking

import com.urbandroid.sleep.domain.AdaptiveNormalizationFilter
import com.urbandroid.util.ScienceUtil
import kotlin.math.max
import kotlin.math.min

class PairActivity(times: AscSortedAppendableArray<Long>, localValues: AppendableArray<Float>, remoteValues: AppendableArray<Float>) {

    private val local = LFSeries(keys = times, values = localValues)
    private val remote = LFSeries(keys = times, values = remoteValues)

    fun aggregate(factor: Int): PairActivity {
        return PairActivity(
                LongAscSortedAppendableArray(initValues = ScienceUtil.decimate(local.keysAsLongArray(), factor)),
                FloatAppendableArray(initValues = ScienceUtil.decimate(ScienceUtil.movingMax(local.valuesAsFloatArray(), factor), factor)),
                FloatAppendableArray(initValues = ScienceUtil.decimate(ScienceUtil.movingMax(remote.valuesAsFloatArray(), factor), factor))
        )
    }

    fun subtractRemoteActivity(renormalize: Boolean): FloatArray {

        //TODO I am using medians here as pivot points for significant activity.
        //Maybe I should use "The Lightsleep Threshold" 3.3.
        //Or something else?
        val loc = local.valuesAsFloatArray()
        val locMed = ScienceUtil.percentile(loc, 50f)
        val rmt = remote.valuesAsFloatArray()
        val rmtMed = ScienceUtil.percentile(rmt, 50f)
        val result = FloatArray(loc.size)

        for(i in 0 until loc.size) {
            val lom = loc[i] - locMed
            if (lom < 0) {
                result[i] = loc[i]
            } else {
                val rom = rmt[i] - rmtMed
                if (rom < 0) {
                    result[i] = loc[i]
                } else {
                    result[i] = locMed + min(max(0f, 2f*lom - rom), lom)
                }
            }
        }

        if (renormalize) {
            return AdaptiveNormalizationFilter.normalizeAmplitudes(result).output
        } else {
            return result
        }
    }

    fun getTimes(): LongArray {
        return local.copyOfKeys().toPrimitiveArray() as LongArray
    }

    fun getLocalActivity(): FloatArray {
        return local.copyOfValues().toPrimitiveArray() as FloatArray
    }

    fun getRemoteActivity(): FloatArray {
        return remote.copyOfValues().toPrimitiveArray() as FloatArray
    }
}

