package com.urbandroid.sleep.notification

import android.annotation.SuppressLint
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.NotificationManager.IMPORTANCE_HIGH
import android.app.NotificationManager.IMPORTANCE_LOW
import android.content.Context
import android.content.Intent
import android.content.Intent.FLAG_ACTIVITY_NEW_TASK
import android.os.Build
import android.provider.Settings.*
import android.support.annotation.RequiresApi
import android.support.v4.app.NotificationManagerCompat
import com.urbandroid.sleep.R
import com.urbandroid.sleep.service.Settings

@SuppressLint("WrongConstant")
@RequiresApi(Build.VERSION_CODES.O)
fun createChannels(context: Context) {

    val notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

    // ALARM
    val alarmText = context.getString(R.string.default_label) + " (Pop up)"
    val alarmChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_ALARM_POP_UP, alarmText, IMPORTANCE_HIGH).apply {
        enableLights(true)
        enableVibration(false)
        setSound(null, null)
    }

    // ALARM
    var alarmNormalText = context.getString(R.string.default_label)
    val alarmNormalChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_ALARM, alarmNormalText, IMPORTANCE_LOW).apply {
        enableLights(false)
        enableVibration(false)
    }

    // BEFORE ALARM
    val beforeAlarmText = context.getString(R.string.before_alarm_notification)
    val beforeAlarmChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_BEFORE_ALARM, beforeAlarmText, IMPORTANCE_LOW).apply {
        enableLights(false)
        enableVibration(false)
    }

    val sleepTrackingText = context.getString(R.string.settings_category_track)
    val sleepTrackingChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_SLEEP_TRACKING, sleepTrackingText, IMPORTANCE_LOW).apply {
        enableLights(false)
        enableVibration(false)
        setSound(null, null)
    }

    val bedtimeText = context.getString(R.string.time_to_bed_title)
    val bedtimeChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_BEDTIME, bedtimeText, IMPORTANCE_HIGH).apply {
        enableLights(true)
        lightColor = 0xFFFFC8C8.toInt()

        val vibrate = Settings(context).isTimeToBedVibrating
        if (vibrate) {
            vibrationPattern = longArrayOf(0, 1000, 1000, 1000, 1000, 1000)
        }
        enableVibration(vibrate)
    }

    val lullabyText = context.getString(R.string.lullaby)
    val lullabyChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_LULLABY, lullabyText, IMPORTANCE_LOW).apply {
        enableLights(false)
        setSound(null, null)
    }

    val backupText = context.getString(R.string.backup)
    val backupChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_BACKUP, backupText, IMPORTANCE_LOW).apply {
        enableLights(false)
        enableVibration(false)
        setSound(null, null)
    }

    val sleepTimeSuggestionText = context.getString(R.string.sleep_time_suggestion)
    val sleepTimeSuggestionChannel = NotificationChannel(NOTIFICATION_CHANNEL_ID_SLEEP_TIME_SUGGESTION, sleepTimeSuggestionText, IMPORTANCE_LOW).apply {
        enableLights(false)
        enableVibration(false)
        setSound(null, null)
    }

    with(notificationManager) {
        createNotificationChannel(alarmChannel)
        createNotificationChannel(alarmNormalChannel)
        createNotificationChannel(beforeAlarmChannel)
        createNotificationChannel(sleepTrackingChannel)
        createNotificationChannel(bedtimeChannel)
        createNotificationChannel(lullabyChannel)
        createNotificationChannel(backupChannel)
        createNotificationChannel(sleepTimeSuggestionChannel)
    }
}

@RequiresApi(Build.VERSION_CODES.O)
fun showSettings(context: Context, channelId: String) {
    val intent = Intent(ACTION_CHANNEL_NOTIFICATION_SETTINGS).apply {
        putExtra(EXTRA_CHANNEL_ID, channelId)
        putExtra(EXTRA_APP_PACKAGE, context.packageName)
        flags = FLAG_ACTIVITY_NEW_TASK
    }

    context.startActivity(intent)
}

fun areNotificationsEnabled(context: Context) = NotificationManagerCompat.from(context).areNotificationsEnabled()

@SuppressLint("NewApi")
fun isNotificationEnabled(context: Context, channelId: String): Boolean {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
        val notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        val channel = notificationManager.getNotificationChannel(channelId)
        if (channel != null && channel.importance != IMPORTANCE_HIGH) {
            return false
        }
    }
    return areNotificationsEnabled(context)
}

const val NOTIFICATION_CHANNEL_ID_ALARM_POP_UP = "alarmChannel"
const val NOTIFICATION_CHANNEL_ID_ALARM = "alarmNormalChannel"
const val NOTIFICATION_CHANNEL_ID_BEFORE_ALARM = "beforeAlarmChannel"
const val NOTIFICATION_CHANNEL_ID_SLEEP_TRACKING = "sleepTrackingChannel" // sleep tracking + antidoze
const val NOTIFICATION_CHANNEL_ID_BACKUP = "backupChannel"
const val NOTIFICATION_CHANNEL_ID_BEDTIME = "bedtimeChannel"
const val NOTIFICATION_CHANNEL_ID_LULLABY = "lullabyChannel"
const val NOTIFICATION_CHANNEL_ID_SLEEP_TIME_SUGGESTION = "sleepTimeSuggestionChannel"
