package com.urbandroid.sleep.sensor.respiration

import com.urbandroid.sleep.domain.CurrentSleepRecord
import com.urbandroid.sleep.domain.SleepRecord
import com.urbandroid.sleep.sensor.respiration.v1.RespiratoryDetectorV1
import com.urbandroid.sleep.sensor.respiration.v1.RespiratoryListenerV1
import com.urbandroid.sleep.sensor.respiration.v2.RespiratoryDetectorV21
import com.urbandroid.sleep.sensor.respiration.v2.RespiratoryListenerV2
import com.urbandroid.sleep.sensor.respiration.v2.androidSleepDataLogger
import com.urbandroid.sleep.sensor.respiration.v2.systemClock
import com.urbandroid.util.Experiments

fun createRespiratoryDetector(sampleRate: Float): RespiratoryDetector {

    val exp = Experiments.getInstance()

    val sleepRecord = CurrentSleepRecord.getInstance().record

    return if (exp.isRespiratoryDetectionV2Experiment) {

        val listener = if (sleepRecord == null)
            RespiratoryDetector.NullRespiratoryListener()
            else
            RespiratoryListenerV2(sleepRecord)

        RespiratoryDetectorV21(
                sampleRate,
                systemClock(),
                if (exp.isRespiratoryDetailsPersistentExperiment) androidSleepDataLogger() else null,
                listener
        )
    }
    else {

        val listener = if (sleepRecord == null)
            RespiratoryDetector.NullRespiratoryListener()
            else
            RespiratoryListenerV1(sleepRecord)

        RespiratoryDetectorV1(sampleRate, listener)
    }
}

fun postProcessRespiratoryEvents(sleepRecord: SleepRecord) {
    val exp = Experiments.getInstance()
    if (exp.isRespiratoryDetectionV2Experiment) {
        com.urbandroid.sleep.sensor.respiration.v2.postProcessRespiratoryEvents(sleepRecord)
    } else {
        RespiratoryDetectorV1.postProcessRespiratoryEvents(sleepRecord)
    }
}
