package com.urbandroid.sleep.sensor.respiration.v2

import android.annotation.TargetApi
import com.urbandroid.util.StringBufferPersister
import java.nio.file.*

interface BreathLogger {
    fun log(s:String)
}

@TargetApi(26)
fun commonFileLogger(file: Path): BreathLogger {
    return object: BreathLogger {
        override fun log(s: String) {
            Files.createDirectories(file.parent)
            Files.write(file, s.toByteArray(), StandardOpenOption.CREATE, StandardOpenOption.APPEND)
        }
    }
}

fun androidSleepDataLogger(): BreathLogger {
    val persister = StringBufferPersister("breath_details", 10000, true);
    return object: BreathLogger {
        override fun log(s: String) {
            persister.update(s)
            persister.flush()
        }
    }
}
