package com.urbandroid.sleep.sensor.respiration.v2

import com.urbandroid.common.logging.Logger
import com.urbandroid.sleep.domain.EventLabel
import com.urbandroid.sleep.domain.EventsUtil
import com.urbandroid.sleep.domain.SleepRecord
import org.apache.commons.math3.stat.StatUtils

fun postProcessRespiratoryEvents(sleepRecord: SleepRecord) {

    Logger.logInfo("RespiratoryDetectorV2: postProcessRespiratoryEvents")

    val events = sleepRecord.events

    val allRespRates = EventsUtil.getEvents(events.copiedEvents, EventLabel.RR)
            .map { it.value.toDouble() }

    if (allRespRates.isNotEmpty()) {

        val mostFrequentRate = StatUtils.mode(allRespRates.toDoubleArray())[0]
        val mostFrequentRateCount = allRespRates.filter { it == mostFrequentRate }.size
        val sameRatesRatio = mostFrequentRateCount.toDouble() / allRespRates.size
        Logger.logInfo("RespiratoryDetectorV2: RR count: " + allRespRates.size + " " + mostFrequentRateCount + " " + sameRatesRatio);

        //If there are too many breaths detected with exactly the same frequency,
        //it is probably not real breath, but just some artifact in the signal,
        //and we remove all the respiration and apnea events.
        //
        //Even if the person breathes absolutely regularly, the measurements
        //always have some variation. The threshold 0.8 is based on analysis of many
        //real measurements, and it is rather optimistic. We might consider lowering
        // it to 0.7 or even 0.6.

        if (sameRatesRatio >= 0.8) {
            Logger.logInfo("RespiratoryDetectorV2: too high sameRatesRatio, removing all respiration-related events. "+sameRatesRatio)
            events.clearLabels(EventLabel.RR, EventLabel.APNEA)
        }
    }
}
